--- windows/input.c	Fri Nov 30 13:46:56 2001
+++ windows/input.c	Tue Jan 15 00:16:04 2002
@@ -14,6 +14,7 @@
 #include <stdio.h>
 #include <ctype.h>
 #include <assert.h>
+#include <sys/timeb.h>
 
 #include "windef.h"
 #include "winnls.h"
@@ -63,6 +64,40 @@
 
 
 /***********************************************************************
+ *           _is_dblclick
+ * Checks if all conditions for a double-click are met
+ */
+BOOL _is_dblclick(int x, int y, int event)
+{
+    static DWORD last_event, last_x, last_y;
+    static double last_tm = 0;
+    double cur_tm;
+    struct timeb now;
+    BOOL ret = FALSE;
+
+    /* calculate time in ms */
+    ftime(&now);
+    cur_tm = now.time * 1000 + now.millitm;
+ 
+    /* x, y, & event must be same and 500 ms apart for dbl click */
+    if (event == last_event && x == last_x && y == last_y &&
+        cur_tm - last_tm <= 500)
+    {
+        last_tm = 0;
+        ret = TRUE;
+    }
+    else
+        last_tm = cur_tm;
+
+    /* save event info */
+    last_event = event;
+    last_x = x;
+    last_y = y;
+
+    return ret;
+}
+
+/***********************************************************************
  *           get_key_state
  */
 static WORD get_key_state(void)
@@ -257,6 +292,10 @@
         AsyncKeyStateTable[VK_LBUTTON] |= 0x80;
         queue_raw_mouse_message( SwappedButtons ? WM_RBUTTONDOWN : WM_LBUTTONDOWN,
                                  flags, PosX, PosY, mi );
+
+        if (_is_dblclick(PosX, PosY, MOUSEEVENTF_LEFTDOWN))
+            queue_raw_mouse_message( SwappedButtons ? WM_RBUTTONDBLCLK :
+                                     WM_LBUTTONDBLCLK, flags, PosX, PosY, mi );
     }
     if (mi->dwFlags & MOUSEEVENTF_LEFTUP)
     {
@@ -270,6 +309,10 @@
         AsyncKeyStateTable[VK_RBUTTON] |= 0x80;
         queue_raw_mouse_message( SwappedButtons ? WM_LBUTTONDOWN : WM_RBUTTONDOWN,
                                  flags, PosX, PosY, mi );
+
+        if (_is_dblclick(PosX, PosY, MOUSEEVENTF_LEFTDOWN))
+            queue_raw_mouse_message( SwappedButtons ? WM_LBUTTONDBLCLK :
+                                     WM_RBUTTONDBLCLK, flags, PosX, PosY, mi );
     }
     if (mi->dwFlags & MOUSEEVENTF_RIGHTUP)
     {
@@ -282,6 +325,9 @@
         InputKeyStateTable[VK_MBUTTON] |= 0x80;
         AsyncKeyStateTable[VK_MBUTTON] |= 0x80;
         queue_raw_mouse_message( WM_MBUTTONDOWN, flags, PosX, PosY, mi );
+
+        if (_is_dblclick(PosX, PosY, MOUSEEVENTF_MIDDLEDOWN))
+            queue_raw_mouse_message( WM_LBUTTONDBLCLK, flags, PosX, PosY, mi );
     }
     if (mi->dwFlags & MOUSEEVENTF_MIDDLEUP)
     {
